const std = @import("std");

pub fn build(b: *std.Build) void {
    const target = b.standardTargetOptions(.{});
    const optimize = b.standardOptimizeOption(.{});

    const httpz = b.dependency("httpz", .{
        .optimize = optimize,
        .target = target,
    });

    const protocol = b.dependency("protocol", .{});

    const exe = b.addExecutable(.{
        .name = "dispatch",
        .root_source_file = b.path("src/main.zig"),
        .target = target,
        .optimize = optimize,
    });

    const tls12 = b.dependency("tls12", .{
        .target = target,
        .optimize = optimize,
    });

    exe.root_module.addImport("tls12", tls12.module("zig-tls12"));
    exe.root_module.addImport("httpz", httpz.module("httpz"));
    exe.root_module.addImport("protocol", protocol.module("protocol"));

    b.installArtifact(exe);

    const run_cmd = b.addRunArtifact(exe);
    run_cmd.step.dependOn(b.getInstallStep());

    if (b.args) |args| {
        run_cmd.addArgs(args);
    }

    const run_step = b.step("run", "Run the app");
    run_step.dependOn(&run_cmd.step);
}
